@@ -5,6 +5,9 @@ sys.path.append('../..')
 import numpy as np
 from MJOLNIR.Geometry import GeometryConcept,Analyser,Detector,Wedge
 from MJOLNIR import _tools
+from MJOLNIR import TasUBlibDEG as TasUBlib
+from MJOLNIR.TasUBlibDEG import cosd,sind
+from MJOLNIR.Data import Sample,RLUAxes
 import warnings
 import matplotlib.pyplot as plt
 from mpl_toolkits.mplot3d import Axes3D
@@ -309,7 +312,8 @@ class Instrument(GeometryConcept.GeometryConcept):
 
     @_tools.KwargChecker()
     def generateCalibration(self,Vanadiumdatafile,A4datafile=False,savelocation='calibration/', 
-    tables=['Single','PrismaticLowDefinition','PrismaticHighDefinition'], plot=False, mask=True, adaptiveBinning=False, ignoreTubes=None):
+    tables=['Single','PrismaticLowDefinition','PrismaticHighDefinition'], plot=False, mask=True, adaptiveBinning=False, ignoreTubes=None,
+    sample='V',sampleMass=None,sampleDebyeWallerFactor=1.0,formulaUnitsPerUnitCell=1.0,sampleIncoherent=5.08):
         """Method to generate look-up tables for normalization. Saves calibration file(s) as 'Calibration_Np.calib', where Np is the number of pixels.
         
         Generates 4 different tables:
@@ -342,9 +346,21 @@ class Instrument(GeometryConcept.GeometryConcept):
 
             - ignoreTubes (list of ints): List containing tubes to be ignored in fitting (default [])
 
+            - sample (string): Chemical composition of sample used for normalization (default 'V')
+
+            - sampleMass (float): Mass of normalization sample (default None)
+
+            - sampleDebyeWallerFactor (float): Correction of normalization data with Debye-Waller factor (default 1.0)
+
+            - formulaUnitsPerUnitCell (float): Numer of formalunits in normalization sample per unit cell (default 1.0) 
+            
+            - sampleIncoherent (float): Incoherent strength of sample (default 5.08)
+
         .. warning::
             At the moment, the active detector area is defined by NumberOfSigmas (currently 3) times the Gaussian width of Vanadium peaks.
 
+        When a sample mass is provided, the normalization tables are per default rescaled with Vanadium unless default parameters are overwritten.
+
         """
         self.initialize()
 
@@ -375,6 +391,7 @@ class Instrument(GeometryConcept.GeometryConcept):
                     savelocation+='/'
                 
                 Data = np.array(VanFileInstrument.get('detector/counts')).transpose(2,0,1).astype(float)
+                monitor = np.mean(np.array(VanFile.get('entry/control/data'))) # Extract monitor count
                 
                 if False: #Mask pixels where spurion is present
                     Data[:,:,:100]=0
@@ -528,6 +545,15 @@ class Instrument(GeometryConcept.GeometryConcept):
                     for j in range(analysers):
                         Eguess[i,j]=np.argmax(Data[i,:,int(pixelpos[i,j])])
                 
+                if sampleMass != None: # A sample mass has been proivided
+
+                    sampleMolarMass = _tools.calculateMolarMass(sample,formulaUnitsPerUnitCell)
+                    
+                    # TODO: Check placement of sampleDebyeWallerFactor!
+                    vanadiumFactor = sampleDebyeWallerFactor*sampleMolarMass/(sampleMass*sampleIncoherent*monitor)
+                else:
+                    vanadiumFactor = 1.0
+
                 fitParameters = []
                 activePixelRanges = []
                 for detpixels in bins:
@@ -614,6 +640,7 @@ class Instrument(GeometryConcept.GeometryConcept):
 
                                     fittedParameters[i,j,k]=res[0]
                                     fittedParameters[i,j,k,0] *= np.sqrt(2*np.pi)*fittedParameters[i,j,k,2] #np.sum(binPixelData) # Use integrated intensity as amplitude 29-07-2020 JL
+                                    fittedParameters[i,j,k,0] *= vanadiumFactor # Multiply vanadium factor to perform absolut normalization 22-02-2021 JL
                                     if plot: # pragma: no cover
                                         plt.plot(EiX,GaussianNormalized(EiX,*fittedParameters[i,j,k]),color='black')
                                         plt.scatter(EiLocal,binPixelData,color=colors[:,k])
@@ -1277,3 +1304,120 @@ def convertToHDF(fileName,title,sample,fname,CalibrationFile=None,pixels=1024,ce
         storeScanData(entry,data,a3,a4,ei,rotation_angle_zero=rotation_angle_zero,polar_angle_offset=polar_angle_offset)
         
 
+def prediction(A3Start,A3Stop,A3Steps,A4Positions,Ei,Cell,r1,r2,points=False):
+    s = Sample.Sample(*Cell,projectionVector1=r1,projectionVector2=r2)
+ 
+    class simpleDataSet():
+        def __init__(self,sample):
+            self.sample = [sample]
+
+   
+    def converterToQxQy(A3,A4,Ei,Ef):
+
+        ki = np.sqrt(Ei)*_tools.factorsqrtEK
+        kf = np.sqrt(Ef)*_tools.factorsqrtEK
+
+        r = [0,0,0,A3,A4,0.0,0.0,Ei,Ef]
+        QV = TasUBlib.calcTasUVectorFromAngles(r)
+        q = np.sqrt(ki**2 +kf**2-
+            2. *ki *kf * np.cos(np.deg2rad(A4)))
+
+        
+        Qx,Qy = (QV*q.reshape(-1,1))[:,:2].T
+        return (Qx,Qy)
+
+    def converterToA3A4(Qx,Qy, Ei,Ef,A3Off=0.0):
+        Qx = np.asarray(Qx)
+        Qy = np.asarray(Qy)
+
+        QC = np.array([Qx,Qy])
+        q = np.linalg.norm(QC)
+
+        U1V = np.array([Qx.flatten(),Qy.flatten(),0.0],dtype=float)
+
+        U1V/=np.linalg.norm(U1V)
+        U2V = np.array([0.0,0.0,1.0],dtype=float)
+        
+        
+        TV = TasUBlib.buildTVMatrix(U1V, U2V)
+        R = np.linalg.inv(TV)
+        
+        ss = 1.0
+        
+        cossgl = np.sqrt(R[0,0]*R[0,0]+R[1,0]*R[1,0])
+        om = TasUBlib.arctan2d(R[1,0]/cossgl, R[0,0]/cossgl)
+     
+        ki = np.sqrt(Ei)*_tools.factorsqrtEK
+        kf = np.sqrt(Ef)*_tools.factorsqrtEK
+        
+        cos2t =(ki**2 + kf**2 - q**2) / (2. * np.abs(ki) * np.abs(kf))
+        
+        A4 = ss*TasUBlib.arccosd(cos2t)
+        theta = TasUBlib.calcTheta(ki, kf, A4)
+        A3 = -om + -ss*theta + A3Off
+        return A3,-A4
+        
+
+    t = simpleDataSet(s)
+    fig = plt.figure(figsize=(26,13))
+    Ax = [RLUAxes.createRLUAxes(t,figure=fig,ids=(3,3,i+1)) for i in range(9)]
+    
+    MJOLNIRDir = os.path.dirname(_tools.__file__)
+    calibPath = os.path.join(MJOLNIRDir,'Normalization_1.calib')
+
+    calib = np.loadtxt(calibPath,delimiter=',',skiprows=3)
+
+    A4Instrument = calib[:,-1].reshape(104,8)
+    EfInstrument = calib[:,4].reshape(104,8)
+    EfInstrument[np.isclose(EfInstrument,0.0)]=np.nan
+
+    A3PointValues = np.linspace(A3Start,A3Stop,A3Steps)-s.theta
+    steps = A3Steps
+    if steps>30:
+        steps = 30
+    A3Values = np.linspace(A3Start,A3Stop,steps)-s.theta
+
+    endColor = np.array([0.12156863, 0.46666667, 0.70588235, 0.5])
+    startColor = np.array([0.83921569, 0.15294118, 0.15686275, 0.5])
+    diff = (endColor-startColor)/7.0
+
+    def format_axes_func(self,x,y,Ei,Ef,A3Off=s.theta):
+        A3,A4 = converterToA3A4(x,y,Ei,Ef,-A3Off)
+        return ax.sample.format_coord(x,y)+', A3 = {:.2f} deg, A4 = {:.2f} deg, Ef = {:.2f}'.format(A3,A4,Ef)
+
+    for i,(ax,Ef,A4) in enumerate(zip(Ax,EfInstrument.reshape(104,8).T,A4Instrument.reshape(104,8).T)):
+        ax.Ef = np.nanmean(Ef)
+        for A4Position in A4Positions:
+            if points:
+                for A3 in A3PointValues:
+                    H,L = converterToQxQy(A3=A3, A4=A4+A4Position,Ei = Ei, Ef = Ef)
+                    ax.scatter(H,L,color=startColor[:-1]+diff[:-1]*i)
+
+            A4Edge = np.array([np.concatenate([a4,[a4[-1]]*steps,a4[::-1],[a4[0]]*steps]) for a4 in A4.reshape(8,13)])
+            EfEdge = np.array([np.concatenate([ef,[ef[-1]]*steps,ef[::-1],[ef[0]]*steps]) for ef in Ef.reshape(8,13)])
+            A3Edge = np.array(list(np.concatenate([[A3Values[0]]*13,A3Values,[A3Values[-1]]*13,A3Values[::-1]]))*8).reshape(8,-1)
+            Hedge,Ledge = np.array([converterToQxQy(A3=a3, A4=a4+A4Position,Ei = Ei, Ef = ef) for a3,a4,ef in zip(A3Edge,A4Edge,EfEdge)]).transpose([1,0,2])
+            [a.plot(hedge,ledge,color=startColor+diff*i) for hedge,ledge in zip(Hedge,Ledge) for a in [Ax[-1],ax]]
+
+            
+        ax.set_xlabel(_tools.generateLabelDirection(s.projectionVector1) + ' RLU')
+        ax.set_ylabel(_tools.generateLabelDirection(s.projectionVector2) + ' RLU')
+        ax.set_title(r'$\Delta $E {:.2f}'.format(Ei-np.nanmean(Ef))+' meV, E$_f$ {:.2f} meV'.format(np.nanmean(Ef)))
+        
+
+    Ax[-1].set_xlabel(_tools.generateLabelDirection(s.projectionVector1) + ' RLU')
+    Ax[-1].set_ylabel(_tools.generateLabelDirection(s.projectionVector2) + ' RLU')
+    Ax[-1].set_title('All Planes')
+
+    fig.tight_layout()
+    Ax[0].format_coord = lambda x,y: format_axes_func(Ax[0],x,y,Ei,np.nanmean(EfInstrument,axis=0)[0],-s.theta)
+    Ax[1].format_coord = lambda x,y: format_axes_func(Ax[1],x,y,Ei,np.nanmean(EfInstrument,axis=0)[1],-s.theta)
+    Ax[2].format_coord = lambda x,y: format_axes_func(Ax[2],x,y,Ei,np.nanmean(EfInstrument,axis=0)[2],-s.theta)
+    Ax[3].format_coord = lambda x,y: format_axes_func(Ax[3],x,y,Ei,np.nanmean(EfInstrument,axis=0)[3],-s.theta)
+    Ax[4].format_coord = lambda x,y: format_axes_func(Ax[4],x,y,Ei,np.nanmean(EfInstrument,axis=0)[4],-s.theta)
+    Ax[5].format_coord = lambda x,y: format_axes_func(Ax[5],x,y,Ei,np.nanmean(EfInstrument,axis=0)[5],-s.theta)
+    Ax[6].format_coord = lambda x,y: format_axes_func(Ax[6],x,y,Ei,np.nanmean(EfInstrument,axis=0)[6],-s.theta)
+    Ax[7].format_coord = lambda x,y: format_axes_func(Ax[7],x,y,Ei,np.nanmean(EfInstrument,axis=0)[7],-s.theta)
+    fig.tight_layout()
+
+    return Ax
\ No newline at end of file
